
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string.h>

#include "L_Types.h"
#include "twodtg_license.h"

#include "LoadBMP.c"

void printLINEAR_QUALITY(LINEAR_QUALITY lq);
void printLINEAR_SCAN_LINE(int idx_scan_line, LINEAR_SCAN_LINE* sl);


#define maxR 4000
#define maxC 4000

int main(int argc, char **argv)
{
  int rowcount = maxR;
  int colcount = maxC;
  TRow*    pbits;
  TRow     pmembits;  // Image in Memory

  PL_Decoder    pDecoder;
  PL_Options    pDecWithOpt;
  TL_ImageInfo* pl_imageinfo;
  TL_Info*      pl_info;
  
  if(argc != 2)
  {
    printf("Usage:\n");
    printf("demo.out MyImageName.bmp\n");
    return 0;
  }
  
  // Check License
   if(IsActivated())
   {
       printf("License status: Fully licensed.\n");
   }else if(EvaluationDayCount() > 0)
   {
       printf("License status: Evaluation expires in %d days.\n.", EvaluationDayCount());
   }else if(RequredRootAcess())
   {
       printf("ATTENTION: ROOT rights are required to initialize a license!\nUse 'sudo' for first run.\n");
       return 0;
   }else
   {
       printf("License status: Evaluation expired!\n");
       return 0;
   }
   
  
  pmembits = (TRow) malloc(maxR*maxC);       //  Image in Memory
  pbits    = (TRow*) malloc(maxR*sizeof(TRow)); // pointers to ScanLines
  for (int row = 0; row < maxR; row++)
  {
      pbits[row] = &pmembits[maxC*row];
  }
  
  // Load the image
  if(LoadBMP (argv[1], pbits, &rowcount, &colcount) != 0)
  {
   free(pmembits);
   free(pbits);
   printf("Failed Load file: %s\n", argv[1]);
   return 0;
  }

  // create Linear decoder 
  pDecoder = Connect_L_Decoder(maxR, maxC);
  
  // create the options
  TL_OptMode opt;
  opt.maxLcount = 10;
  opt.typeCode  = TC_ANY; // All codes
  opt.paramQ    = 1;      // calc paramq;
  opt.checkSum_I25 = 0;   // off checksum_I25;
  opt.checkSum_C39 = 0;   // off checksum_C39;
  opt.checkSum_CB = 0;    // off checksum_CB;
  opt.FullASCII_C39 = 0;  // FullASCII_C39;
  opt.quietZone = -1;     // quietzone;
  opt.smartmode = 0; //1; // smartmode
  opt.EC_Factor = 0;      // EC_Factor
  opt.pharmacodedir = 0;  // pharmacodedir
  opt.scandir   = 2;      // scandir = vertical+horizontal;
  opt.scanstep  = 8;      // scanstep;
  opt.colbeg    = -1;     // colbeg
  opt.colend    = -1;     // colend
  opt.rowbeg    = -1;     // rowbeg
  opt.rowend    = -1;     // rowend
  opt.TADF_num  = 99;     // TADF_num
  opt.color     = L_DEFAULT; // color any
  opt.mode      = L_M_STANDARD; // Mode of decoding
                     
  pDecWithOpt = Create_L_Options(pDecoder, opt);

  Decode_L_Bits(pDecWithOpt, rowcount, colcount, pbits);

  pl_imageinfo = GetL_ImageInfo(pDecWithOpt);   
  if((pl_imageinfo->LCount > 0)) 
  {
     printf("Barcodes count: %d\n", pl_imageinfo->LCount);
     
     for(int i = 0; i < pl_imageinfo->LCount; i++) 
     {
       printf("Barcode %d:\n", i + 1);
       pl_info = GetL_Info(pDecWithOpt, i);
       
       if(pl_info->RejectionReason == L_SUCCESSFUL) 
         printf("SUCCESSFUL\n");
       else if (pl_info->RejectionReason == L_CHECKSUM_FAILED) 
         printf("CHECKSUM_FAILED\n");
       else if (pl_info->RejectionReason == L_NOT_CONFIRMED) 
         printf("NOT_CONFIRMED\n");
       else if (pl_info->RejectionReason == L_NOT_QUIET_ZONE) 
         printf("NOT_QUIET_ZONE\n");
         
       switch (pl_info->type)
       {
         case TC_TCODE128     : printf("Type: C128\n"); 
          break;
         case TC_TCODABAR     : printf("Type: Cbar\n"); 
          break;
         case TC_TCODE39      : printf("Type: C39\n"); 
          break;
         case TC_TINTERLEAVED : printf("Type: Il 2/5\n"); 
          break;
         case TC_TEAN13       : printf("Type: EAN13\n"); 
          break;
         case TC_TEAN8        : printf("Type: EAN8\n"); 
          break;
         case TC_TUPCE        : printf("Type: UPCE\n"); 
          break;
         case TC_TEAN13EX     : printf("Type: EAN13_ADDON\n"); 
          break;
         case TC_TUPCEEX      : printf("Type: UPCE_ADDON\n"); 
          break;
         case TC_TPHARMACODE  : printf("Type: Pharma\n"); 
          break;
         case TC_TADF  : printf("Type: TADF, barcount=%d\n", pl_info->add_lq->actTADFnum);
          break;
       };

       if(pl_info->pchlen > 0)
       {
        if (pl_info->pch[-3] == ']')
        {
         printf("Identifier: %c%c%c\n", pl_info->pch[-3], pl_info->pch[-2], pl_info->pch[-1]);
        }
       
        printf("Data: %s\n", pl_info->pch);
       }
       printLINEAR_SCAN_LINE(pl_info->idx_scan_line, pl_info->sl);

       printf("\n\n");
     }
     
     printLINEAR_QUALITY(pl_info->lq);
  }
  else 
  {  // lq==0
     switch(pl_imageinfo->RejectionReason) 
     {
       case L_NO_LINEAR_FOUND:  printf("NO_LINEAR_FOUND!\n");
        break;
       case L_POOR_IMAGE_QUALITY:  printf("POOR_IMAGE_QUALITY!\n"); 
        break;
       case L_RR_UNKNOWN:   printf("Unknown error !\n"); 
        break;
     }
  }

  Delete_L_Options (pDecWithOpt);
  Disconnect_L_Decoder(pDecoder);

  free(pbits);
  free(pmembits);
  return 0;
}

void printLINEAR_QUALITY(LINEAR_QUALITY lq)
{
  printf("\nGrades for the scan reflectance profile\n");
  printf("Parameter    Value   Grade          Average Grade on ten scannings\n");
  printf("Decode                 %d                     %6.2f\n",(int)lq.decode_grad,lq.decode_grad_av);
  printf("Rmax         %6.2f%%                           \n",lq.max_reflectance  );
  printf("Rmin         %6.2f%%   %d                     %6.2f\n",lq.min_reflectance  ,(int)lq.min_reflectance_grad,lq.min_reflectance_grad_av);
  printf("SC           %6.2f%%   %d                     %6.2f\n",lq.symbol_contrast  ,(int)lq.symbol_contrast_grad,lq.symbol_contrast_grad_av);
  printf("ECmin        %6.2f%%   %d                     %6.2f\n",lq.min_edge_contrast,(int)lq.min_edge_contrast_grad,lq.min_edge_contrast_grad_av);
  printf("MOD          %6.2f    %d                     %6.2f\n",lq.modulation       ,(int)lq.modulation_grad,lq.modulation_grad_av);
  printf("Defects      %6.2f    %d                     %6.2f\n",lq.defects          ,(int)lq.defects_grad,lq.defects_grad_av);
  printf("Decodability %6.2f    %d                     %6.2f\n",lq.decodability     ,(int)lq.decodability_grad,lq.decodability_grad_av);
  printf("Overall Grade          %d                     %6.2f\n",                     (int)lq.overall_grade_grad,lq.overall_grade_grad_av);
}

void printLINEAR_SCAN_LINE(int idx_scan_line, LINEAR_SCAN_LINE* sl) 
{
  printf("\nResult for the ten scan reflectance profile\n");
  printf(" N start_x  start_y  end_x  end_y  q_zone  c_sum  length  data\n");
  for(int i=0; i<10; i++) {
    if(i != idx_scan_line)
      printf(" %d %6d  %7d  %5d  %5d  %6d  %5d  %5d   %s\n",i,
                                            sl[i].scan_line_start_x,
                                            sl[i].scan_line_start_y,
                                            sl[i].scan_line_end_x,
                                            sl[i].scan_line_end_y,
                                            sl[i].quietzone,
                                            sl[i].checksum,
                                            sl[i].length,
                                            sl[i].dataRes);
    else
      printf(" %d * %4d * %6d * %4d * %4d * %5d * %4d * %4d * %s\n",i,
                                            sl[i].scan_line_start_x,
                                            sl[i].scan_line_start_y,
                                            sl[i].scan_line_end_x,
                                            sl[i].scan_line_end_y,
                                            sl[i].quietzone,
                                            sl[i].checksum,
                                            sl[i].length,
                                            sl[i].dataRes);
  }
}

